@@ -1,6 +1,5 @@
 from typing import Dict, Optional, cast
 import backoff
-import json
 
 from google.ads.googleads.client import GoogleAdsClient
 from google.ads.googleads.errors import GoogleAdsException
@@ -9,10 +8,15 @@ from google.oauth2.credentials import Credentials
 from arcane.core.exceptions import GOOGLE_EXCEPTIONS_TO_RETRY, BadRequestError
 from arcane.core.types import BaseAccount
 from arcane.datastore import Client as DatastoreClient
-from arcane.secret import decrypt
 
-from .exceptions import GoogleAdsAccountLostAccessException
-from .helpers import get_google_ads_account, _get_google_ads_user_crendential
+from .exceptions import GoogleAdsAccountLostAccessException, GoogleAdsAccountNoAccessException, GoogleAdsAccountException
+from .helpers import (
+    get_google_ads_account,
+    remove_dash,
+    _get_user_initialized_client,
+    _init_datastore_client,
+    get_login_customer_id_and_developer_token
+)
 
 
 _GOOGLE_ADS_VERSION = "v7"
@@ -24,7 +28,7 @@ def get_exception_message(account_id: str, access_token: Optional[str] = None) -
     else:
         return F"We cannot access your account with the id: {account_id} from the Arcane Manager Account. Are you sure you granted access and gave the correct ID?"
 
-# TODO: Delete access token logic when AMCC is finished
+# TODO: Delete access token, developer token and login_customer_id logic when AMCC is finished
 def get_google_ads_client(
     mcc_credentials_path: Optional[str] = None,
     access_token: Optional[str] = None,
@@ -57,7 +61,6 @@ def get_google_ads_client(
         gcp_project: (Optional[str], optional): Needed for getting google_ads_user_credential. Defaults to None.
         secret_key_file: (Optional[str], optional): Needed for decrypt google_ads_user_credential. Defaults to None.
     Raises:
-        ValueError
         arcane.core.BadRequestError
 
     Returns:
@@ -79,28 +82,31 @@ def get_google_ads_client(
                 raise BadRequestError('clients_service_url or firebase_api_key or  gcp_credentials_path should not be None if google_ads_account is not provided')
             base_account = cast(BaseAccount, base_account)
             google_ads_account = get_google_ads_account(
-                base_account,
-                clients_service_url,
-                firebase_api_key,
-                gcp_credentials_path,
-                auth_enabled
+                base_account=base_account,
+                clients_service_url=clients_service_url,
+                firebase_api_key=firebase_api_key,
+                gcp_credentials_path=gcp_credentials_path,
+                auth_enabled=auth_enabled
             )
         creator_email = google_ads_account.get('creator_email')
         if creator_email:
+            login_customer_id, developer_token = get_login_customer_id_and_developer_token(mcc_credentials_path)
             if not secret_key_file:
                 raise BadRequestError('secret_key_file should not be None while using user access protocol')
             elif not developer_token:
                 raise BadRequestError('developer_token should not be None while using user access protocol')
-            user_credentials = _get_google_ads_user_crendential(creator_email, datastore_client, gcp_credentials_path, gcp_project)
-            credentials = decrypt(user_credentials['credentials'], secret_key_file).decode('utf-8')
-            credentials = Credentials(
-                **json.loads(credentials)
-            )
-            return GoogleAdsClient(
-                credentials,
-                developer_token
+            return _get_user_initialized_client(
+                user_email=creator_email,
+                secret_key_file=secret_key_file,
+                developer_token=developer_token,
+                gcp_credentials_path=gcp_credentials_path,
+                gcp_project=gcp_project,
+                datastore_client=datastore_client
             )
         return GoogleAdsClient.load_from_storage(mcc_credentials_path)
+    # TODO: Clean this elif when updating check access acount in AMCC track
+    elif mcc_credentials_path:
+        return GoogleAdsClient.load_from_storage(mcc_credentials_path)
     else:
         raise ValueError('one of the following arguments must be specified: access_token or (mcc_credentials_path and (google_ads_account or google_ads_account_id))')
 
@@ -120,7 +126,7 @@ def check_access_account(
 
     google_ads_client = get_google_ads_client(adscale_key, access_token, login_customer_id, developer_token)
     google_ads_service = get_google_ads_service('GoogleAdsService', google_ads_client)
-
+    account_id = remove_dash(account_id)
     query = f"""
         SELECT
           customer_client.manager
@@ -146,3 +152,97 @@ def check_access_account(
 
     if response.customer_client.manager:
         raise GoogleAdsAccountLostAccessException('This account ID is a MCC. Please enter a Google Ads Account.')
+
+
+def check_access_before_creation(
+    account_id: str,
+    user_email: str,
+    secret_key_file: str,
+    gcp_credentials_path: str,
+    gcp_project: str,
+    login_customer_id: str,
+    developer_token: str,
+) -> bool:
+    """Check if user identified by its email and our mcc have access or not to an account
+
+    Args:
+        account_id (str): Account to check if user has access
+        user_email (str): user email
+        secret_key_file (str): Path to rsa key to decrypt user credentials stored in datastore
+        gcp_credentials_path (str): Needed for getting credentials stored in datastore
+        gcp_project (str): Needed for getting credentials stored in datastore
+        login_customer_id (str): additionnal info needed in client side oauth pattern
+        developer_token (str): developer token for accessing the API.
+
+    Raises:
+        GoogleAdsAccountNoAccessException
+        GoogleAdsException
+
+    Returns:
+        bool: our mcc does it have access
+    """
+    account_id = remove_dash(account_id)
+    mcc_access = False
+    user_access = False
+
+    datastore_client = _init_datastore_client(gcp_credentials_path, gcp_project)
+
+    ## First step: Check if user have direct access
+    try:
+        client_initialized_with_user_direct_access = _get_user_initialized_client(
+            user_email=user_email,
+            secret_key_file=secret_key_file,
+            developer_token=developer_token,
+            gcp_credentials_path=gcp_credentials_path,
+            gcp_project=gcp_project,
+            datastore_client=datastore_client
+        )
+        _check_access_account_lgq(account_id, client_initialized_with_user_direct_access)
+        user_access = True
+    except GoogleAdsAccountNoAccessException:
+        pass
+
+    ## Second step: Check if user have access via our mcc (meaning that our mcc have access)
+    try:
+        client_initialized_with_user_access_via_mcc = _get_user_initialized_client(
+            user_email=user_email,
+            secret_key_file=secret_key_file,
+            developer_token=developer_token,
+            gcp_credentials_path=gcp_credentials_path,
+            gcp_project=gcp_project,
+            datastore_client=datastore_client,
+            login_customer_id=login_customer_id
+        )
+        _check_access_account_lgq(account_id, client_initialized_with_user_access_via_mcc)
+        mcc_access = True
+    except GoogleAdsAccountNoAccessException:
+        if not user_access:
+            raise GoogleAdsAccountNoAccessException("You don't have access to this Google Ads account, you cannot link it to SmartFeeds. Please ask someone else who have access to do the linking")
+    return mcc_access
+
+
+def _check_access_account_lgq(google_ads_account_id: str, google_ads_client: GoogleAdsClient):
+    google_ads_service = get_google_ads_service('GoogleAdsService', google_ads_client)
+
+    query = f"""
+        SELECT
+          customer.manager
+        FROM customer
+        WHERE customer.id = '{google_ads_account_id}'"""
+    search_query = google_ads_client.get_type(
+        "SearchGoogleAdsRequest"
+    )
+    search_query.customer_id = google_ads_account_id
+    search_query.query = query
+    try:
+        response = list(google_ads_service.search(search_query))
+        if len(response) == 0:
+            raise GoogleAdsAccountException(f"We cannot find this account ({google_ads_account_id}). Are you sure you entered the correct id?")
+        response = response[0]
+    except (GoogleAdsException, GoogleAdsAccountException) as err:
+        if "USER_PERMISSION_DENIED" in str(err):
+            raise GoogleAdsAccountNoAccessException
+        else:
+            raise GoogleAdsAccountException(f"We cannot find this account ({google_ads_account_id}). Are you sure you entered the correct id?")
+    if response.customer_client.manager:
+        raise GoogleAdsAccountException('This account ID is a MCC. Please enter a Google Ads Account.')
